@@ -36,6 +36,7 @@
 /* uncomment the next line to get poll() debugging info */
 /* #define G_MAIN_POLL_DEBUG */
 
+#include "galias.h"
 #include "glib.h"
 #include "gthreadinit.h"
 #include <signal.h>
@@ -265,7 +266,7 @@ enum {
   CHILD_WATCH_INITIALIZED_THREADED
 };
 static gint child_watch_init_state = CHILD_WATCH_UNINITIALIZED;
-static gint child_watch_count = 0;
+static gint child_watch_count = 1;
 static gint child_watch_wake_up_pipe[2] = {0, 0};
 #endif /* !G_OS_WIN32 */
 G_LOCK_DEFINE_STATIC (main_context_list);
@@ -588,14 +589,18 @@ g_poll (GPollFD *fds,
  * @context: a #GMainContext
  * 
  * Increases the reference count on a #GMainContext object by one.
+ *
+ * Returns: the @context that was passed in (since 2.6)
  **/
-void
+GMainContext *
 g_main_context_ref (GMainContext *context)
 {
-  g_return_if_fail (context != NULL);
-  g_return_if_fail (g_atomic_int_get (&context->ref_count) > 0); 
+  g_return_val_if_fail (context != NULL, NULL);
+  g_return_val_if_fail (g_atomic_int_get (&context->ref_count) > 0, NULL); 
 
   g_atomic_int_inc (&context->ref_count);
+
+  return context;
 }
 
 /* If DISABLE_MEM_POOLS is defined, then freeing the
@@ -710,7 +715,7 @@ g_main_context_init_pipe (GMainContext *context)
 }
 
 void
-_g_main_thread_init ()
+_g_main_thread_init (void)
 {
   GSList *curr = main_contexts_without_pipe;
   while (curr)
@@ -731,7 +736,7 @@ _g_main_thread_init ()
  * Return value: the new #GMainContext
  **/
 GMainContext *
-g_main_context_new ()
+g_main_context_new (void)
 {
   GMainContext *context = g_new0 (GMainContext, 1);
 
@@ -3565,12 +3570,17 @@ g_child_watch_signal_handler (int signum)
 static void
 g_child_watch_source_init_single (void)
 {
+  struct sigaction action;
+
   g_assert (! g_thread_supported());
   g_assert (child_watch_init_state == CHILD_WATCH_UNINITIALIZED);
 
   child_watch_init_state = CHILD_WATCH_INITIALIZED_SINGLE;
 
-  signal (SIGCHLD, g_child_watch_signal_handler);
+  action.sa_handler = g_child_watch_signal_handler;
+  sigemptyset (&action.sa_mask);
+  action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
+  sigaction (SIGCHLD, &action, NULL);
 }
 
 static gpointer
@@ -3618,6 +3628,7 @@ static void
 g_child_watch_source_init_multi_threaded (void)
 {
   GError *error = NULL;
+  struct sigaction action;
 
   g_assert (g_thread_supported());
 
@@ -3630,7 +3641,11 @@ g_child_watch_source_init_multi_threaded (void)
   if (g_thread_create (child_watch_helper_thread, NULL, FALSE, &error) == NULL)
     g_error ("Cannot create a thread to monitor child exit status: %s\n", error->message);
   child_watch_init_state = CHILD_WATCH_INITIALIZED_THREADED;
-  signal (SIGCHLD, g_child_watch_signal_handler);
+ 
+  action.sa_handler = g_child_watch_signal_handler;
+  sigemptyset (&action.sa_mask);
+  action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
+  sigaction (SIGCHLD, &action, NULL);
 }
 
 static void
